# Locally defined/include autoconf macros that won't get clobbered by
# running aclocal and automake

AC_DEFUN([TYPE_SOCKLEN_T],
[AC_CACHE_CHECK([for socklen_t], ac_cv_type_socklen_t,
[
  AC_TRY_COMPILE([#include <sys/types.h>
   #include <sys/socket.h>],
  [socklen_t len = 42; return 0;],
  ac_cv_type_socklen_t=yes,
  ac_cv_type_socklen_t=no)
])
  if test $ac_cv_type_socklen_t != yes; then
    AC_DEFINE(socklen_t, int, [Substitute for socklen_t])
  fi
])


# Check to see if the given source standard specification #define can
# be included on this platform without breaking unistd.h... if so,
# then add an AC_DEFINE() for it.
AC_DEFUN([CHECK_SOURCE_STANDARDS_DEFINE],
[AC_CACHE_CHECK( [if $1 can be defined to $2], [_cv_can_use_$1],
 [AC_TRY_COMPILE([#define $1 $2
#include <unistd.h>],[int x = 0;],
                 _cv_can_use_$1=yes,
                 _cv_can_use_$1=no)
 ])
 if test $_cv_can_use_$1 = yes; then
   AC_DEFINE([$1],[$2],[$3])
 fi
])


# See if we can statically link a C++ program on this platform
AC_DEFUN([CHECK_STATIC],
 [AC_CACHE_CHECK([if we can statically link on this platform],
                 [_cv_have_static],
 [_save_LDFLAGS=$LDFLAGS
  LDFLAGS="$LDFLAGS -static"
  AC_LINK_IFELSE(AC_LANG_PROGRAM(
[#if HAVE_NETDB_H
#include <netdb.h>
#endif],
                 [struct hostent *foo = gethostbyname("foo");]),
                 [_cv_have_static=yes],
                 [_cv_have_static=no])
  LDFLAGS="$_save_LDFLAGS";
 ])
 if test $_cv_have_static = "yes" ; then
    AC_DEFINE([HAVE_STATIC],[1],[Can we statically link on this platform])
 fi
])


AC_DEFUN([CHECK_RECV_FLAG],
 [AC_CACHE_CHECK([for $1],[_cv_have_$1],
 [
   AC_LINK_IFELSE(AC_LANG_PROGRAM(
     [#if HAVE_SYS_TYPES_H
      #include <sys/types.h>
      #endif
      #if HAVE_SYS_SOCKET_H
      #include <sys/socket.h>
      #endif
     ],
     [int flag = $1;]),
     _cv_have_$1=yes,
     _cv_have_$1=no)
 ])
 if test x$_cv_have_$1 = xyes ; then
    AC_DEFINE([HAVE_$1],[1],[Do we have the $1 recv() flag on this platform])
 fi
])

