/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"
#include "client.h"
#include "GCB_errno.h"

// Global data structure
extern int _CB_initialized;
extern struct FDinfo *_CB_fdInfo;
extern time_t _CB_lastHeartBeat;
extern int _CB_maxfd;
extern int _GCB_max_no_fds;


static void
copySock(int fd, int newfd)
{
	struct SockInfo *info = _CB_fdInfo[fd].info;

	LOG(D_FULL, "copySock(%d to %d)\n", fd, newfd);

	// increase reference count of _CB_socket
	info->refs++;
	// SockInfo->fd has always the smallest fd
	if (info->owner > newfd) {
		info->owner = newfd;
	}
	LOG(D_FULL, "copySock: info->refs = %d\n", info->refs);
	// register newfd
	_CB_fdInfo[newfd].type = _CB_fdInfo[fd].type;
	_CB_fdInfo[newfd].info = info;
	// adjust _CB_maxfd if necessary
	if (_CB_maxfd <= newfd) {
		_CB_maxfd = newfd + 1;
	}
}

int
GCB_dup(int fd)
{
	int newfd;

	if (!_CB_initialized) { _CB_initialize(); }

    if (fd < 0 || fd >= _GCB_max_no_fds || _CB_fdInfo[fd].type == NOT_DEFINED) {
        newfd = dup(fd);
        _GCB_errno = (newfd < 0) ? GCB_ESYSCALLFAILED : 0;
		return dup(fd);
	}

	if (_CB_fdInfo[fd].type == MANAGER || _CB_fdInfo[fd].type == HIDDEN) {
		LOG(D_BASIC,"GCB_dup() called with internal GCB socket");
		errno = EBADF;
        _GCB_errno = GCB_EINTERNALSOCK;
		return -1;
	}

	LOG(D_FULL, "GCB_dup(%d)\n", fd);
	// dup CB_socket
	newfd = dup(fd);
	if (newfd < 0) {
		ERROR("GCB_dup: dup(%d) failed", fd);
        _GCB_errno = GCB_ESYSCALLFAILED;
		return -1;
	}

	copySock(fd, newfd);
    _GCB_errno = 0;
	return newfd;
}

int
GCB_dup2(int fd, int newfd)
{
    int rst;

	if (!_CB_initialized) { _CB_initialize(); }

	// TODO We can handle this better. We can GCB_close() a TCP or UDP
	//   type fd, and dup() a MANAGER or HIDDEN fd to another fd.
	if (newfd >= 0 && newfd < _GCB_max_no_fds && _CB_fdInfo[newfd].type != NOT_DEFINED) {
		LOG(D_BASIC, "GCB_dup2(%d, %d): attempting to dup over an fd GCB cares about. Returning error\n", fd, newfd);
		errno = EBADF;
		_GCB_errno = GCB_EINTERNALSOCK;
		return -1;
	}

    if (fd < 0 || fd >= _GCB_max_no_fds || _CB_fdInfo[fd].type == NOT_DEFINED) {
        rst = dup2(fd, newfd);
        _GCB_errno = (rst < 0) ? GCB_ESYSCALLFAILED : 0;
        return rst;
	}

	if (_CB_fdInfo[fd].type == MANAGER || _CB_fdInfo[fd].type == HIDDEN) {
		LOG(D_BASIC,"GCB_dup2() called with internal GCB socket\n");
		errno = EBADF;
		_GCB_errno = GCB_EINTERNALSOCK;
		return -1;
	}

	LOG(D_FULL, "GCB_dup2(%d, %d)\n", fd, newfd);
	// dup CB_socket
    rst = dup2(fd, newfd);
	if (rst < 0) {
		ERROR("GCB_dup2: dup2() failed");
        _GCB_errno = GCB_ESYSCALLFAILED;
		return -1;
	}

	copySock(fd, newfd);
    _GCB_errno = 0;
	return rst;
}

/*
int
GCB_fcntl(int fd, int cmd)
{
	int newfd, rst;
	struct SockInfo *info;

	if (!_CB_initialized) { _CB_initialize(); }
	if(cmd != F_DUPFD) {
		return fcntl(fd, cmd);
	}

	info = _CB_fdInfo[fd].info;
	if (!info) {
		return fcntl(fd, cmd);
	}

	LOG(D_FULL, "GCB_fcntl(%d, %d)\n", fd, newfd);
	// dup CB_socket
	rst = fcntl(fd, cmd);
	if (rst < 0) {
		return rst;
	}

	if (copySock(fd, newfd) < 0) {
		close(newfd);
		return -1;
	}

	return rst;
}

int
GCB_fcntl(int fd, int cmd, long arg)
{
	int newfd, rst;
	struct SockInfo *info;

	if (!_CB_initialized) { _CB_initialize(); }
	if(cmd != F_DUPFD) {
		return fcntl(fd, cmd, arg);
	}

	info = _CB_fdInfo[fd].info;
	if (!info) {
		return fcntl(fd, cmd, arg);
	}

	LOG(D_FULL, "GCB_fcntl(%d, %d, %ld)\n", fd, newfd, arg);
	// dup CB_socket
	rst = fcntl(fd, cmd, arg);
	if (rst < 0) {
		return rst;
	}

	if (copySock(fd, newfd) < 0) {
		close(newfd);
		return -1;
	}

	return rst;
}
*/
