/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"
#include "client.h"
#include "GCB_errno.h"

// Global data structure
extern int _CB_initialized;
extern struct FDinfo *_CB_fdInfo;
extern time_t _CB_lastHeartBeat;
extern int _GCB_max_no_fds;


int
GCB_getsockname(int fd, struct sockaddr * name, socklen_t *namelen)
{
	struct SockInfo *info;
	struct sockaddr_in *sin;
    int rst;

	if (!_CB_initialized) { _CB_initialize(); }

    if (fd < 0 || fd >= _GCB_max_no_fds || _CB_fdInfo[fd].type == NOT_DEFINED) {
        rst = getsockname(fd, name, namelen);
        _GCB_errno = (rst < 0) ? GCB_ESYSCALLFAILED : 0;
        return rst;
	}

	if (_CB_fdInfo[fd].type == MANAGER || _CB_fdInfo[fd].type == HIDDEN) {
        LOG(D_BASIC, "GCB_real_getsockname(%d, %x, %d): File descriptor passed in is a GCB internal FD that should not be visible to user code.  Returning error...\n", fd, name, (int)namelen, _GCB_max_no_fds);
        errno = EBADF;
        _GCB_errno = GCB_EINTERNALSOCK;
		return -1;
	}
	info = _CB_fdInfo[fd].info;

	//LOG(D_FULL, "[GCB_getsockname(%d)]\n", fd);
	sin = (struct sockaddr_in *) name;
	sin->sin_family = AF_INET;
	*namelen = sizeof(struct sockaddr_in);
	sin->sin_addr.s_addr = info->cip;
	sin->sin_port = info->cport;
	//LOG(D_FULL, "\t%s returned\n", _sin2str(sin));
    _GCB_errno = 0;
	return 0;
}

int
GCB_real_getsockname(int fd, struct sockaddr * name, socklen_t *namelen)
{
	int last_wrapper_status;
	int rst;
	if( ! useGCB() ) {
		return getsockname(fd, name, namelen);
	}

	if (fd >= 0 && fd < _GCB_max_no_fds && 
		(_CB_fdInfo[fd].type == MANAGER || _CB_fdInfo[fd].type == HIDDEN)) {

        LOG(D_BASIC, "GCB_real_getsockname(%d, %x, %d): File descriptor passed in is a GCB internal FD that should not be visible to user code.  Returning error...\n", fd, name, (int)namelen, _GCB_max_no_fds);
        errno = EBADF;
        _GCB_errno = GCB_EINTERNALSOCK;
		return -1;
	} 

	last_wrapper_status = _GCB_disable_wrappers(1);
	rst = getsockname(fd, name, namelen);
	_GCB_disable_wrappers(last_wrapper_status);

	_GCB_errno = (rst < 0) ? GCB_ESYSCALLFAILED : 0;
	return rst;
}
