/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"
#include "client.h"
#include "GCB_errno.h"

// Global data structure
extern int _CB_initialized;
extern struct FDinfo *_CB_fdInfo;
extern time_t _CB_lastHeartBeat;
extern int _GCB_max_no_fds;

int
GCB_getsockopt(
		int fd,
		int level,
		int optname,
		void *opt_val,
		socklen_t *optlen)
{
	struct SockInfo *info;
	int err;
    int rst;

	if (!_CB_initialized) { _CB_initialize(); }

    if (fd < 0 || fd >= _GCB_max_no_fds || _CB_fdInfo[fd].type == NOT_DEFINED) {
        rst = getsockopt(fd, level, optname, opt_val, optlen);
        _GCB_errno = (rst < 0) ? GCB_ESYSCALLFAILED : 0;
        return rst;
	}

	if (_CB_fdInfo[fd].type == MANAGER || _CB_fdInfo[fd].type == HIDDEN) {
		LOG(D_BASIC, "GCB_getsockopt called for internal GCB socket\n");
		errno = EBADF;
        _GCB_errno = GCB_EINTERNALSOCK;
		return -1;
	}

	LOG(D_FULL, "GCB_getsockopt(%d)\n", fd);
	if (level != SOL_SOCKET || optname != SO_ERROR) {
        if (!getsockopt(fd, level, optname, opt_val, optlen)) {
            _GCB_errno = 0;
            return 0;
        } else {
            _GCB_errno = GCB_ESYSCALLFAILED;
            return -1;
        }
    }

	info = _CB_fdInfo[fd].info;
	switch(info->status) {
		case PASSIVE_CONNECTING:
			err = EINPROGRESS;
			memcpy(opt_val, &err, sizeof(int));
			*optlen = sizeof(int);
            _GCB_errno = 0;
			return 0;
		case CONN_FAILED:
			if (info->err == 0) {
				ERROR("GCB_getsockopt(%d): status is CONN_FAILED "
					  "but errno = 0, returning ECONNREFUSED", fd);
				err = ECONNREFUSED;
				memcpy(opt_val, &err, sizeof(int));
				*optlen = sizeof(int);
			} else {
				memcpy(opt_val, &info->err, sizeof(int));
				*optlen = sizeof(int);
			}
            _GCB_errno = 0;
			return 0;
		default: // CONNECTED, ACTIVE_CONNECTING, BOUND, CREATED, LISTENING
            if (!getsockopt(fd, level, optname, opt_val, optlen)) {
                _GCB_errno = 0;
                return 0;
            } else {
                _GCB_errno = GCB_ESYSCALLFAILED;
                return -1;
            }
	}
}
