/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"
#include "client.h"

extern int _CB_maxfd;
extern struct FDinfo *_CB_fdInfo;
extern time_t _CB_lastHeartBeat;
extern time_t _CB_lastFlush;
extern void (*_Broker_down_app_callback)(void);
extern time_t _Broker_down_max;
extern time_t _Broker_down_since;

void
_CB_triggerHeartBeat(time_t current)
{
	int i, needrecover = 0;
	struct SockInfo *info;

	LOG(D_FULL, "-----Triggering Heartbeat (time: %d)-----\n", (int)current);
	/* Send HeartBeat to Broker */
	for (i = 0; i < _CB_maxfd; i++) {
		info = _CB_fdInfo[i].info;
		if (_CB_fdInfo[i].type == NOT_DEFINED || // not a GCB socket
			_CB_fdInfo[i].type == HIDDEN || // internally-accepted connection
			_CB_fdInfo[i].type == MANAGER || // manager
			i != info->owner || // duped socket. HEARTBEAT will be sent by the owner
			info->manager == NO_MNG || // not registered to the Broker
            info->manager == NEED_RECOVER
		) {
			continue;
		}
		if (_CBsend(info->manager, HEARTBEAT, 0, 0, 0, 0, 0, 0, 0, 0) < 0) {
			ERROR("_CB_triggerHeartBeat: can't send HEARTBEAT for the client: "
				  "_CBsend failed");
			_CB_dumpSockInfo(_CB_fdInfo[i].info, D_BASIC, "\t");
            _CB_rmManager(info->manager);
            info->manager = NEED_RECOVER;
			needrecover = 1;
		}
	}

	if (needrecover) {
        if (_Broker_down_since != 0) {
            _Broker_down_since = time(NULL);
        }
		return _CB_recoverGCBsockets();
	}

	/* Less frequently, we also flush receiver list of UDP socket, because there is no way,
	   for UDP, to know whether the peer socket has been closed or not
	 */
	if (current - _CB_lastFlush > GCB_FLUSH_RATE) {
		for (i = 0; i < _CB_maxfd; i++) {
			if (_CB_fdInfo[i].type != UDP) continue;
			info = _CB_fdInfo[i].info;
			if (info->rcvers) {
				/*
				LOG(D_FULL,
				    "_CB_triggerHeartBeat: Deleting rcver[fd = %d]\n", i);
				_CB_dumpSockInfo(_CB_fdInfo[i].info, D_FULL, "\t");
				*/
				_CB_delRcvrs(info);
			}
		}
		_CB_lastFlush = current;
	}
	LOG(D_FULL, "-----Heartbeat Done-----\n");
}
