/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"
#include "client.h"
#include <signal.h>
#include <setjmp.h>

extern int _CB_initialized;
extern fd_set _CB_fds;
extern fd_set _CB_mfds;
extern int _CB_maxfd;
extern struct FDinfo *_CB_fdInfo;
extern time_t _CB_lastCheck;
extern time_t _CB_lastHeartBeat;
extern time_t _CB_lastFlush;
extern int _GCB_max_no_fds;

/* Initialize data structure and connect to CB if necessary */
// #include <CBclient.h>
// @return: abort if failed 
void
_CB_initialize()
{
	int i;
	struct sigaction act;
    char *logpath, *env;
    int loglevel, maxlog;
    int inherit_max_fd;
	GCBString inheritString;

    // init log
    logpath = getenv("GCB_LOG_FILE");
    if (logpath) {
        loglevel = D_BASIC;
        env = getenv("GCB_LOG_LEVEL");
        if (env) {
            if (!strcasecmp(env, "fulldebug")) { loglevel = D_FULL; }
        }
        maxlog = 64000;
        env = getenv("GCB_LOG_MAXSIZE");
        if (env) { maxlog = atol(env); }
        Generic_logInit(logpath, loglevel, maxlog);
    }

	inheritString = _CB_inheritFromString();

	LOG(D_FULL, "++++++++++++++++++++++++++++++\n");
	LOG(D_FULL, "+ _CB_initialize() called\n");
    // Get the max # of fds per process
    // We can inherit FDs larger than getdtablesize(), so check for those
    // in the inherit list and make our arrays large enough to accommodate
    // them.
    _GCB_max_no_fds = getdtablesize();
    inherit_max_fd = _CB_inheritFromMaxFD(inheritString);
    if ( inherit_max_fd >= _GCB_max_no_fds ) {
        _GCB_max_no_fds = inherit_max_fd + 1;
        _CB_setMaxNumFds( _GCB_max_no_fds );
    }

	// Initialize Data Structure
    _CB_fdInfo = (struct FDinfo *) malloc ( sizeof(struct FDinfo) * _GCB_max_no_fds );
    if (!_CB_fdInfo) { ABORT("malloc failed"); }
	for (i = 0; i < _GCB_max_no_fds; i++) {
		_CB_fdInfo[i].type = NOT_DEFINED;
		_CB_fdInfo[i].info = NULL;
	}
    LOG(D_FULL, "_CB_fdInfo[0..%d] has been initialized\n", _GCB_max_no_fds-1);
	// Let the process ignore SIGPIPE
	act.sa_handler = SIG_IGN;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	if (sigaction(SIGPIPE, &act, NULL) < 0) {
		ERROR("_CB_initialize: sigaction(SIGPIPE) failed");
	}

	// inherit CB_sockets, if any
	_CB_inheritFrom(inheritString);
	GCB_string_delete(inheritString);
	LOG(D_FULL, "++++++++++++++++++++++++++++++\n");
	_CB_initialized = 1;
	return;
}
