/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"
#include "client.h"
#include "GCB_errno.h"

// Global data structure
extern int _CB_initialized;
extern struct FDinfo *_CB_fdInfo;
extern int _GCB_max_no_fds;

int
GCB_send(int fd, const void *msg, size_t len, int flags)
{
	int rst, fcntl_mode;

	if (!_CB_initialized) { _CB_initialize(); }

    if (fd < 0 || fd >= _GCB_max_no_fds || _CB_fdInfo[fd].type == NOT_DEFINED) {
		rst = send( fd, msg, len, flags );
		_GCB_errno = (rst < 0) ? GCB_ESYSCALLFAILED : 0;
		return rst;
    }

	if (_CB_fdInfo[fd].type == MANAGER || _CB_fdInfo[fd].type == HIDDEN) {
		LOG(D_FULL, "GCB_send called with internal GCB socket\n");
		errno = EBADF;
		_GCB_errno = GCB_EINTERNALSOCK;
		return -1;
	}

	LOG(D_FULL, "GCB_send (fd=%d, size=%d)\n", fd, len);

	if( ! _GCB_is_blocking(fd,flags,&fcntl_mode) ) {
			// Nonblocking send or UDP send returns immediately
		rst = send(fd, msg, len, flags);
        LOG(D_FULL, "GCB_send: returning %d\n", rst);
        if (rst < 0 && errno != EAGAIN) {
            _GCB_errno = GCB_ESYSCALLFAILED;
        } else {
            _GCB_errno = 0;
        }
		return rst;
	}
    rst = _GCB_nonblock_send( fd, msg, len, flags, fcntl_mode );
    if (rst < 0) {
        _GCB_errno = GCB_ESYSCALLFAILED;
        return -1;
    } else {
        _GCB_errno = 0;
        return rst;
    }
}


int
GCB_write(int fd, const void *buf, size_t count)
{
	int rst;
	int fcntl_mode;

	if (!_CB_initialized) { _CB_initialize(); }

    if (fd < 0 || fd >= _GCB_max_no_fds || _CB_fdInfo[fd].type == NOT_DEFINED) {
		rst = write(fd, buf, count);
		_GCB_errno = (rst < 0) ? GCB_ESYSCALLFAILED : 0;
		return rst;
    }

	if (_CB_fdInfo[fd].type == MANAGER || _CB_fdInfo[fd].type == HIDDEN) {
		LOG(D_FULL, "GCB_write called with internal GCB socket\n");
		errno = EBADF;
		_GCB_errno = GCB_EINTERNALSOCK;
		return -1;
	}

	LOG(D_FULL, "GCB_write (fd=%d, size=%d)\n", fd, count);

	if( ! _GCB_is_blocking(fd,0,&fcntl_mode) ) {
			// socket FD is non-blocking, use the real write()...
		rst = write(fd, buf, count);
        if (rst < 0 && errno != EAGAIN) {
            _GCB_errno = GCB_ESYSCALLFAILED;
        } else {
            _GCB_errno = 0;
        }
        LOG(D_FULL, "GCB_write: returning %d\n", rst);
		return rst;
	}

		// socket FD would block, call our non-blocking send()...
	rst = _GCB_nonblock_send( fd, buf, count, 0, fcntl_mode );
    if (rst < 0) {
        _GCB_errno = GCB_ESYSCALLFAILED;
        return -1;
    } else {
        _GCB_errno = 0;
        return rst;
    }
}
