/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#define GCB_MAX_MSG_STR 1000

static int
printOP(char opcode, char *str)
{
	switch(opcode) {
		case RELAY_SERVER:
			sprintf(str, "RELAY_SERVER: ");
			break;
		case ACK:
			sprintf(str, "ACK");
			break;
		case NAK:
			sprintf(str, "NAK: ");
			break;
        case CONNECT:
            sprintf(str, "CONNECT: ");
            break;
        case INIT_COMM:
            sprintf(str, "INIT_COMM: ");
            break;
        case LISTEN:
            sprintf(str, "LISTEN: ");
            break;
        case ACTIVE_INIT:
            sprintf(str, "ACTIVE_INIT: ");
            break;
        case TRIGGER:
            sprintf(str, "TRIGGER: ");
            break;
		case TRIGGER_FWD:
            sprintf(str, "TRIGGER_FWD: ");
            break;
        case CONTACT:
            sprintf(str, "CONTACT: ");
            break;
		case CONTACT_FWD:
            sprintf(str, "CONTACT_FWD: ");
            break;
        case ACTIVE_CONNECT:
            sprintf(str, "ACTIVE_CONNECT: ");
            break;
        case BIND_OK:
            sprintf(str, "BIND_OK: ");
            break;
		case HEARTBEAT:
            sprintf(str, "HEARTBEAT");
            break;
        case PASSIVE_INIT:
            sprintf(str, "PASSIVE_INIT");
            break;
        case PASSIVE_CONNECT:
            sprintf(str, "PASSIVE_CONNECT");
            break;
        case DUMMY:
            sprintf(str, "DUMMY: ");
            break;
        case DUMMY_ACK:
            sprintf(str, "DUMMY_ACK: ");
            break;
        case TCP_BIND:
            sprintf(str, "TCP_BIND: ");
            break;
        case UDP_BIND:
            sprintf(str, "UDP_BIND: ");
            break;
		case TCP_DEL:
            sprintf(str, "TCP_DEL: ");
            break;
		case UDP_DEL:
            sprintf(str, "UDP_DEL: ");
            break;
		case TCP_CLIENT:
            sprintf(str, "TCP_CLIENT: ");
            break;
		case UDP_CLIENT:
            sprintf(str, "UDP_CLIENT: ");
            break;
		case NEW_SOCK:
            sprintf(str, "NEW_SOCK");
            break;
		case DATA_QUERY:
            sprintf(str, "DATA_QUERY: ");
            break;
		case DATA_RESPONSE:
            sprintf(str, "DATA_RESPONSE");
            break;
		case REMOTE_REPORT:
            sprintf(str, "REMOTE_REPORT");
            break;
		default:
			LOG(D_BASIC, "Unexpected opcode %d passed to printOP\n", (int)opcode);
            sprintf(str, "UNKNOWN OP %d", (int)opcode);
			return -1;
    }
	return 0;
}

/* Stripe a msg with given arguments */
// This routine should always succeed. If something bad was passed from the caller, this function
// will abort the process.
// TODO: pass in sizeof(buf).  if(sizeof(buf) < size) {ABORT(...);}
unsigned short
_CB_stripeMsg(
		char *buf,
		char opcode,
		uint32_t cip,
		uint16_t cport,
		char active,
		uint32_t ip,
		uint16_t port,
		uint32_t dip,
		uint16_t dport,
		char errcode)
{
	unsigned short size = 0;
	uint16_t nsize;
	char str[GCB_MAX_MSG_STR];

	printOP(opcode, str);

	switch(opcode) {
		case RELAY_SERVER:
			size = 9;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &ip, 4);
			memcpy(&buf[7], &port, 2);
			sprintf(&str[strlen(str)], "addr=%s ", _ipport2str(ip, port));
			break;
		case TCP_BIND:
		case UDP_BIND:
		case TCP_CLIENT:
		case UDP_CLIENT:
			size = 16;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &cip, 4);
			memcpy(&buf[7], &cport, 2);
			sprintf(&str[strlen(str)], "official=%s ", _ipport2str(cip, cport));
			memcpy(&buf[9], &active, 1);
			if (active == ACTIVE) {
				strcat(str, "A ");
			} else if (active == PASSIVE) {
				strcat(str, "P ");
			} else {
				strcat(str, "N ");
			}
			memcpy(&buf[10], &ip, 4);
			memcpy(&buf[14], &port, 2);
			sprintf(&str[strlen(str)], "local=%s ", _ipport2str(ip, port));
			break;
		case BIND_OK:
		case LISTEN:
		case TCP_DEL:
		case UDP_DEL:
			size = 9;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &cip, 4);
			memcpy(&buf[7], &cport, 2);
			sprintf(&str[strlen(str)], "canon=%s ", _ipport2str(cip, cport));
			break;
		case CONNECT:
		case INIT_COMM:
			size = 22;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &cip, 4);
			memcpy(&buf[7], &cport, 2);
			sprintf(&str[strlen(str)], "canon=%s ", _ipport2str(cip, cport));
			memcpy(&buf[9], &active, 1);
			if (active == ACTIVE) {
				strcat(str, "A ");
			} else if (active == PASSIVE) {
				strcat(str, "P ");
			}
			memcpy(&buf[10], &ip, 4);
			memcpy(&buf[14], &port, 2);
			sprintf(&str[strlen(str)], "phyAddr=%s ", _ipport2str(ip, port));
			memcpy(&buf[16], &dip, 4);
			memcpy(&buf[20], &dport, 2);
			sprintf(&str[strlen(str)], "dest=%s ", _ipport2str(dip, dport));
			break;
		case ACTIVE_CONNECT:
		case CONTACT:
		case ACTIVE_INIT:
			size = 9;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &ip, 4);
			memcpy(&buf[7], &port, 2);
			sprintf(&str[strlen(str)], "phyAddr=%s ", _ipport2str(ip, port));
			break;
		case PASSIVE_CONNECT:
		case PASSIVE_INIT:
		case ACK:
		case HEARTBEAT:
		case NEW_SOCK:
			size = 3;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			break;
		case CONTACT_FWD:
		case TRIGGER:
			size = 15;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &cip, 4);
			memcpy(&buf[7], &cport, 2);
			sprintf(&str[strlen(str)], "canon=%s ", _ipport2str(cip, cport));
			memcpy(&buf[9], &ip, 4);
			memcpy(&buf[13], &port, 2);
			sprintf(&str[strlen(str)], "phyAddr=%s ", _ipport2str(ip, port));
			break;
		case TRIGGER_FWD:
			size = 21;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &cip, 4);
			memcpy(&buf[7], &cport, 2);
			sprintf(&str[strlen(str)], "canon=%s ", _ipport2str(cip, cport));
			memcpy(&buf[9], &ip, 4);
			memcpy(&buf[13], &port, 2);
			sprintf(&str[strlen(str)], "phyAddr=%s ", _ipport2str(ip, port));
			memcpy(&buf[15], &dip, 4);
			memcpy(&buf[19], &dport, 2);
			sprintf(&str[strlen(str)], "client=%s", _ipport2str(dip, dport));
			break;
		case DUMMY:
		case DUMMY_ACK: // cip:cport -> dip:dport
			size = 15;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &cip, 4);
			memcpy(&buf[7], &cport, 2);
			sprintf(&str[strlen(str)], "%s -> ", _ipport2str(cip, cport));
			memcpy(&buf[9], &dip, 4);
			memcpy(&buf[13], &dport, 2);
			sprintf(&str[strlen(str)], "%s", _ipport2str(dip, dport));
			break;
		case NAK:
			size = 4;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &errcode, 1);
			if (errcode == PEER_NOT_REGISTERED) {
				sprintf(&str[strlen(str)], "-> PEER_NOT_REGISTERED ");
			} else {
				sprintf(&str[strlen(str)], "-> %s", strerror(errcode));
			}
			break;
		case DATA_QUERY:
			size = 7;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &cip, 4);
			sprintf(&str[strlen(str)], "query=%d ", cip);
			break;
		case DATA_RESPONSE:
			size = 11;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &ip, 4);
			memcpy(&buf[7], &cip, 4);
			sprintf(&str[strlen(str)], "query=%d resp=%d", ip, cip);
			break;
		case REMOTE_REPORT:
			/* This is a (sort of) generic reporting tool designed
			   for extensibility, so we shove almost everything down the wire.
			   (We can't include errcode since old brokers only
			   accept messages up to 22 bytes long and hang up if you try.)
			   (Sadly it's too late to do this for the entire protocol,
			   which would dramatically simplify things.)
			*/
			size = 22;
			nsize = htons(size);
			memcpy(buf, &nsize, 2);
			memcpy(&buf[2], &opcode, 1);
			memcpy(&buf[3], &cip, 4);
			memcpy(&buf[7], &cport, 2);
			sprintf(&str[strlen(str)], "type=%d message=", (int)active);
			sprintf(&str[strlen(str)], "%d %d (%s) ", (int)cip, (int)cport, _ipport2str(cip, cport));
			memcpy(&buf[9], &active, 1);
			memcpy(&buf[10], &ip, 4);
			memcpy(&buf[14], &port, 2);
			sprintf(&str[strlen(str)], "%d %d (%s) ", (int)ip, (int)port, _ipport2str(ip, port));
			memcpy(&buf[16], &dip, 4);
			memcpy(&buf[20], &dport, 2);
			sprintf(&str[strlen(str)], "%d %d (%s) ", (int)dip, (int)dport, _ipport2str(dip, dport));
			break;
		default:
			ABORT("_CB_stripeMsg: invalid opcode passed\n"); // TODO INTERNAL
	}
	LOG(D_FULL, "*send* %s\n", str);
	return size;
}

static int validate_opcode_length(int size, int validsize, char opcode, char * opcodestr) {
	if(size < validsize) {
		LOG(D_BASIC, "Received an invalid GCB message. (The opcode %s (%d) requires at least %d bytes of data, but only %d were provided.",
			opcodestr, (int)opcode, validsize, size);
		return 0;
	}
	return 1;
}

/* Parse _CB msg which is "size" bytes long and stored in "buf" */
// @ret: 0 if succeed
//		 -1 if failed
// TODO: Validate that nobytes size is >= correct. (or just == for paranoia, at risk of less backward compatibility in the future)
// TODO: Perhaps have a shared function that returns correct size for a given opcode.  Then stripeMsg and unstripemsg could share it.
int
_CB_unstripeMsg(
		int nobytes,
		char *buf,
		char *opcode,
		uint32_t *cip,
		uint16_t *cport,
		char *active,
		uint32_t *ip,
		uint16_t *port,
		uint32_t *dip,
		uint16_t *dport,
		char *errcode)
{
	uint16_t size;
	char str[GCB_MAX_MSG_STR];

	memcpy(&size, buf, 2);
	size = ntohs(size);
	if (size != nobytes) {
		//LOG(D_BASIC, "_CB_unstripeMsg: size field is different from the msg size\n");
		// TODO: This probably represents a serious internal consistancy error, worthy of at least a scary message.  Why was the above commented out?
		return -1;
	}

	if(size < 3) {
		LOG(D_BASIC, "Received an invalid GCB message. "
			"(_CB_unstripeMsg: Message is %d bytes, shorter than minimum of %d.)\n",
			size, 3);
		return -1;
	}

	*opcode = buf[2];
	printOP(*opcode, str);

	switch(*opcode) {
		case RELAY_SERVER:
			if( ! validate_opcode_length(size, 9, *opcode, str) ) { return -1; }
			memcpy(ip, &buf[3], 4);
			memcpy(port, &buf[7], 2);
			sprintf(&str[strlen(str)], "addr=%s ", _ipport2str(*ip, *port));
			break;
		case TCP_BIND:
		case UDP_BIND:
		case TCP_CLIENT:
		case UDP_CLIENT:
			if( ! validate_opcode_length(size, 16, *opcode, str) ) { return -1; }
			memcpy(cip, &buf[3], 4);
			memcpy(cport, &buf[7], 2);
			sprintf(&str[strlen(str)], "canon=%s ", _ipport2str(*cip, *cport));
			memcpy(active, &buf[9], 1);
			if (*active == ACTIVE) {
				strcat(str, "A ");
			} else if (*active == PASSIVE) {
				strcat(str, "P ");
			} else {
				strcat(str, "N ");
			}
			memcpy(ip, &buf[10], 4);
			memcpy(port, &buf[14], 2);
			sprintf(&str[strlen(str)], "phyAddr=%s ", _ipport2str(*ip, *port));
			break;
		case BIND_OK:
		case LISTEN:
		case TCP_DEL:
		case UDP_DEL:
			if( ! validate_opcode_length(size, 9, *opcode, str) ) { return -1; }
			memcpy(cip, &buf[3], 4);
			memcpy(cport, &buf[7], 2);
			sprintf(&str[strlen(str)], "canon=%s ", _ipport2str(*cip, *cport));
			break;
		case CONNECT:
		case INIT_COMM:
			if( ! validate_opcode_length(size, 22, *opcode, str) ) { return -1; }
			memcpy(cip, &buf[3], 4);
			memcpy(cport, &buf[7], 2);
			sprintf(&str[strlen(str)], "canon=%s ", _ipport2str(*cip, *cport));
			memcpy(active, &buf[9], 1);
			if (*active == ACTIVE) {
				strcat(str, "A ");
			} else if (*active == PASSIVE) {
				strcat(str, "P ");
			} else {
				strcat(str, "N ");
			}
			memcpy(ip, &buf[10], 4);
			memcpy(port, &buf[14], 2);
			sprintf(&str[strlen(str)], "phyAddr=%s ", _ipport2str(*ip, *port));
			memcpy(dip, &buf[16], 4);
			memcpy(dport, &buf[20], 2);
			sprintf(&str[strlen(str)], "dest=%s ", _ipport2str(*dip, *dport));
			break;
		case ACTIVE_CONNECT:
		case CONTACT:
		case ACTIVE_INIT:
			if( ! validate_opcode_length(size, 9, *opcode, str) ) { return -1; }
			memcpy(ip, &buf[3], 4);
			memcpy(port, &buf[7], 2);
			sprintf(&str[strlen(str)], "phyAddr=%s ", _ipport2str(*ip, *port));
			break;
		case PASSIVE_CONNECT:
		case PASSIVE_INIT:
		case ACK:
		case HEARTBEAT:
		case NEW_SOCK:
			break;
		case CONTACT_FWD:
		case TRIGGER:
			if( ! validate_opcode_length(size, 15, *opcode, str) ) { return -1; }
			memcpy(cip, &buf[3], 4);
			memcpy(cport, &buf[7], 2);
			sprintf(&str[strlen(str)], "canon=%s ", _ipport2str(*cip, *cport));
			memcpy(ip, &buf[9], 4);
			memcpy(port, &buf[13], 2);
			sprintf(&str[strlen(str)], "phyAddr=%s ", _ipport2str(*ip, *port));
			break;
		case TRIGGER_FWD:
			if( ! validate_opcode_length(size, 21, *opcode, str) ) { return -1; }
			memcpy(cip, &buf[3], 4);
			memcpy(cport, &buf[7], 2);
			sprintf(&str[strlen(str)], "canon=%s ", _ipport2str(*cip, *cport));
			memcpy(ip, &buf[9], 4);
			memcpy(port, &buf[13], 2);
			sprintf(&str[strlen(str)], "phyAddr=%s ", _ipport2str(*ip, *port));
			memcpy(dip, &buf[15], 4);
			memcpy(dport, &buf[19], 2);
			sprintf(&str[strlen(str)], "client=%s ", _ipport2str(*dip, *dport));
			break;
		case DUMMY:
		case DUMMY_ACK: // cip:cport -> dip:dport
			if( ! validate_opcode_length(size, 15, *opcode, str) ) { return -1; }
			memcpy(cip, &buf[3], 4);
			memcpy(cport, &buf[7], 2);
			sprintf(&str[strlen(str)], "%s -> ", _ipport2str(*cip, *cport));
			memcpy(dip, &buf[9], 4);
			memcpy(dport, &buf[13], 2);
			sprintf(&str[strlen(str)], "%s", _ipport2str(*dip, *dport));
			break;
		case NAK:
			if( ! validate_opcode_length(size, 4, *opcode, str) ) { return -1; }
			memcpy(errcode, &buf[3], 1);
			if (*errcode == PEER_NOT_REGISTERED) {
				sprintf(&str[strlen(str)], "-> PEER_NOT_REGISTERED ");
			} else {
				sprintf(&str[strlen(str)], "-> %s", strerror(*errcode));
			}
			break;
		case DATA_QUERY:
			if( ! validate_opcode_length(size, 7, *opcode, str) ) { return -1; }
			memcpy(cip, &buf[3], 4);
			sprintf(&str[strlen(str)], "query=%d ", *cip);
			break;
		case DATA_RESPONSE:
			if( ! validate_opcode_length(size, 11, *opcode, str) ) { return -1; }
			memcpy(ip, &buf[3], 4);
			memcpy(cip, &buf[7], 4);
			sprintf(&str[strlen(str)], "query=%d resp=%d", *ip, *cip);
			break;

		case REMOTE_REPORT:
			if( ! validate_opcode_length(size, 22, *opcode, str) ) { return -1; }
			memcpy(cip, &buf[3], 4);
			memcpy(cport, &buf[7], 2);
			memcpy(active, &buf[9], 1);
			sprintf(&str[strlen(str)], "type=%d message=", (int)active);
			sprintf(&str[strlen(str)], "%d %d (%s) ", (int)*cip, (int)*cport, _ipport2str(*cip, *cport));
			memcpy(ip, &buf[10], 4);
			memcpy(port, &buf[14], 2);
			sprintf(&str[strlen(str)], "%d %d (%s) ", (int)*ip, (int)*port, _ipport2str(*ip, *port));
			memcpy(dip, &buf[16], 4);
			memcpy(dport, &buf[20], 2);
			sprintf(&str[strlen(str)], "%d %d (%s) ", (int)*dip, (int)*dport, _ipport2str(*dip, *dport));
			break;

		default:
			LOG(D_BASIC, "GCB received an unknown message. (_CB_unstripeMsg: unknown opcode %d)\n", (int)*opcode);
			return -1;
	}
	LOG(D_FULL, "*recv* %s\n", str);
	return 0;
}

// @ret: size of the msg sent, if success
//       -1, if error
int
_CBsend(int fd,
		char opcode,
		uint32_t cip,
		uint16_t cport,
		char active,
		uint32_t ip,
		uint16_t port,
		uint32_t dip,
		uint16_t dport,
		char errcode)
{
	char buf[50];
	unsigned short size;
	int remained;
	int sent;
	size = _CB_stripeMsg(buf, opcode, cip, cport, active, ip, port, dip, dport, errcode);
	remained = size;
	while (remained > 0) {
		sent = write(fd, buf, remained);
		if (sent < 0) {
			char opcodebuf[25];
			char cbuf[24], dbuf[24]; /* only one call to _ipport2str in a given LOG */
			if (errno == EINTR) continue;
			printOP(opcode, opcodebuf);
			_ipport2strbuf(cip, cport, cbuf);
			_ipport2strbuf(dip, dport, dbuf);
			LOG(D_BASIC, "Failed to write() a message: %s. (_CBsend(fd: %d, opcode: %s(%u), c: %s (cip: %u, cport: %u), active: %d, ipport: %s (ip: %u, port: %u), d: %s (dip: %u, dport: %u), errorcode: %d))\n", strerror(errno), fd, opcodebuf, (unsigned int)opcode, cbuf, (unsigned int)cip, (unsigned int)cport, (int)active, _ipport2str(ip,port), (unsigned int)ip, (unsigned int)port, dbuf, (unsigned int)dip, (unsigned int)dip, (int)errcode);
			return -1;
		}
		remained = remained - sent;
	}
	return size;
}


/* timeout is in seconds. */
static struct timeval gs_GCB_timeout;
static int gs_GCB_timeout_initialized = 0;
#define DEFAULT_GCB_TIMEOUT 10.0

void initialize_timeout() {
	double timeout_d;
	char* timeout_str;
	if( gs_GCB_timeout_initialized ) { return; }

	timeout_str = getenv("GCB_COMMAND_TIMEOUT");
	if(timeout_str && (atof(timeout_str) > 0.0)) {
		timeout_d = atof(timeout_str);
	} else {
		timeout_d = DEFAULT_GCB_TIMEOUT;
	}
	gs_GCB_timeout.tv_sec = (long)timeout_d;
	timeout_d -= gs_GCB_timeout.tv_sec;
	gs_GCB_timeout.tv_usec = (long)(timeout_d*1000000);
	gs_GCB_timeout_initialized = 1;
	LOG(D_FULL, "Command read timeout (GCB_COMMAND_TIMEOUT) set to %d.%06d seconds.\n", gs_GCB_timeout.tv_sec, gs_GCB_timeout.tv_usec);
}

// @ret: size of the msg received, if success
//		 0, if the connection was closed
//       -1, if error
int
_CBrecv(int fd,
		char *opcode,
		uint32_t *cip,
		uint16_t *cport,
		char *active,
		uint32_t *ip,
		uint16_t *port,
		uint32_t *dip,
		uint16_t *dport,
		char *errcode)
{
	char buf[50];
	int size;
	uint16_t nsize;
	int recvd;
	int MAX_CBRECV_SIZE = 22;
	int MIN_CBRECV_SIZE = 3; /* two byte length, plus one byte command.*/

	initialize_timeout();

	// receive the 1st 2 bytes
	recvd = read_with_timeout(fd, &nsize, 2, gs_GCB_timeout);
	if (recvd < 0) {
		return -1;
	} else if (recvd < 2) {
		return 0;
	}
	memcpy(buf, &nsize, 2);
	size = ntohs(nsize);
	if (size > MAX_CBRECV_SIZE) {
		LOG(D_BASIC, "Received an invalid GCB message. "
			"(_CBrev: Message is %d bytes, larger than limit of %d.\n",
			size, MAX_CBRECV_SIZE);
		return -1;
	}

	if(size < MIN_CBRECV_SIZE) {
		LOG(D_BASIC, "Received an invalid GCB message. "
			"(_CBrev: Message is %d bytes, shorter than minimum of %d.\n",
			size, MIN_CBRECV_SIZE);
		return -1;
	}

	// receive the remaining part
	  // 2 = the two bytes read above.
	recvd = read_with_timeout(fd, &buf[2], size - 2, gs_GCB_timeout);
	if (recvd < 0) {
		return -1;
	} else if (recvd < (size - 2)) {
		return 0;
	}

	// unstripe the msg
	if (_CB_unstripeMsg(size, buf, opcode, cip, cport, active, ip, port, dip, dport, errcode) < 0) {
		char hexbuf[101];
		int i;
		for(i = 0; i < size; i++) {
			sprintf(&hexbuf[i*2], "%02x", (int)buf[i]);
		}
		LOG(D_BASIC, "Received an invalid GCB message. (_CBrecv: _CB_unstripeMsg failed to parse message \"%s\"\n", hexbuf);
		return -1;
	}
	return size;
}
