/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include <stdlib.h>
#include <string.h>
#include "gcbstring.h"


GCBString GCB_string_new(const char * p) {
	GCBString s = (GCBString)malloc(sizeof(struct GCBStringI));
	if(p == NULL) {
		p = "";
	}
	s->strlen = strlen(p);
	s->buflen = s->strlen+1;
	s->s = strdup(p);
	return s;
}

GCBString GCB_string_new_empty(size_t bufsize) {
	GCBString s = (GCBString)malloc(sizeof(struct GCBStringI));
	/* Ensure we can at least have a trailing null */
	if(bufsize < 1) { bufsize = 1; }
	s->strlen = 0;
	s->buflen = bufsize;
	s->s = malloc(bufsize);
	s->s[0] = 0;
	return s;
}


void GCB_string_delete(GCBString s) {
	s->s[0] = 0;
	free(s->s);
	s->s = NULL;
	s->strlen = 0;
	s->buflen = 0;
	free(s);
}

void GCB_string_truncate(GCBString s) {
	s->s[0] = 0;
	s->strlen = 0;
}

size_t GCB_string_length(GCBString s) { return s->strlen; }

const char * GCB_string_c_str(GCBString s) { return s->s; }

void GCB_string_trim_last(GCBString s) {
	if(s->strlen > 0) {
		s->s[s->strlen - 1] = 0;
		s->strlen--;
	}
}

/*
Note that srclen MUST BE CORRECT.  This function will happily walk
off the end of src, ignoring NULL characters, if srclen says so.
*/
void GCG_string_appendGPlen(GCBString dest, const char * src, size_t srclen) {
	if( (dest->strlen + srclen + 1) > dest->buflen) {
		size_t bigbuf = dest->buflen;
		if(srclen > bigbuf) { bigbuf = srclen; }
		bigbuf *= 2;
		bigbuf += 1;
		dest->s = (char*)realloc(dest->s, bigbuf);
		dest->buflen = bigbuf;
	}
	memcpy(dest->s + dest->strlen, src, srclen);
	dest->strlen += srclen;
	dest->s[dest->strlen] = 0; /* Ensure null termination */
	return;
}


void GCB_string_appendGG(GCBString dest, GCBString src) {
	GCG_string_appendGPlen(dest, src->s, src->strlen);
}

void GCB_string_appendGP(GCBString dest, const char * src) {
	if(src == NULL) { src = ""; }
	GCG_string_appendGPlen(dest, src, strlen(src));
}



/* Testing code. */
#if 0
void dumpstr(GCBString s) {
	size_t maxlen = (s->strlen > s->buflen)?s->strlen:s->buflen;
	size_t i;
	printf("%s\n", GCB_string_c_str(s));
	for(i = 1; i <= maxlen; i++) {
		if(i == s->strlen && i == s->buflen) { printf(":"); }
		else if(i == s->strlen) { printf("'"); }
		else if(i == s->buflen) { printf("."); }
		else { printf(" "); }
	}
	printf("(%d, %d)\n", (int)s->strlen, (int)s->buflen);
}

int main(int argc, char ** argv) {
	GCBString s = GCB_string_new("Test1"); dumpstr(s);
	GCBString s2 = GCB_string_new("AnotherString");
	GCB_string_appendGP(s, "!"); dumpstr(s);
	GCB_string_appendGP(s, "@"); dumpstr(s);
	GCB_string_appendGP(s, "#$"); dumpstr(s);
	GCB_string_appendGP(s, "abcd"); dumpstr(s);
	GCB_string_appendGP(s, "XYZ"); dumpstr(s);
	GCB_string_trim_last(s); dumpstr(s);
	GCB_string_trim_last(s); dumpstr(s);
	GCB_string_appendGP(s, "1234567890"); dumpstr(s);
	GCB_string_appendGP(s, "A"); dumpstr(s);
	GCB_string_appendGP(s, NULL); dumpstr(s);
	GCB_string_appendGG(s, s2); dumpstr(s);
	GCB_string_appendGP(s, ""); dumpstr(s);
	GCB_string_clear(s); dumpstr(s);
	GCB_string_appendGP(s, "0987654321"); dumpstr(s);
	GCB_string_appendGP(s, "zyxw"); dumpstr(s);

	GCB_string_delete(s2);
	GCB_string_delete(s);

	return 0;
}
#endif
