/****************************************************************************
 * Please read "../../LICENSE.TXT" for copyright
 ****************************************************************************/
#include "gcb_common.h"

// This file is far from complete
// The main goal at this moment is that Generic socket can log using Condor's
// dprint and this file just provide a place that Condor's dprintf can be plugged in

int Generic_log_initialized = 0;
int Generic_log_enabled = 1;
char Generic_logfile[PATH_MAX];
int Generic_debug_level = D_BASIC;
int Generic_maxlog = 64000;
char *Generic_log_srcfile;
int Generic_log_lineno;
// the pointer to the actual log function
static void default_log_va(int level, char *fmt, va_list ap);
void (*Generic_log_va)(int flag, char *fmt, va_list args) = default_log_va;

void
Generic_set_log_va(void(*app_log_va)(int level, char *fmt, va_list args))
{
	Generic_log_va = app_log_va;
}

void
Generic_logInit(char *path, int debug_level, int maxsize)
{
	char msg[1000];
	char timestamp[64];
	int fd;
	struct tm *t, *localtime();
    time_t clock;

	Generic_log_initialized = 1;

	// Compose a logfile name and open the logfile
	if (path) {
        // The path cannot be equal to or bigger than PATH_MAX - 4
        if (strlen(path) >= PATH_MAX-4) { return; }
		strcpy(Generic_logfile, path);
		fd = open(Generic_logfile, O_RDWR | O_CREAT | O_APPEND, 0666 );
		if (fd < 0){
			Generic_logfile[0] = '\0';
			return;
		}
	} else {
		return;
	}

	// Set log level
	Generic_debug_level = debug_level;
	// Set the maximum size of the logfile
	Generic_maxlog = maxsize;

	(void)time(  (time_t *)&clock );
    t = localtime( (time_t *)&clock );

	snprintf( timestamp, sizeof(timestamp), "%d/%d %02d:%02d:%02d",
			  t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec );

	snprintf( msg, sizeof(msg), 
			  "%s ****************************************\n"
			  "%s New log file started\n"
			  "%s Max size = %d\n"
			  "%s Log level: %s\n"
			  "%s ****************************************\n",
			  timestamp, timestamp, timestamp, Generic_maxlog, timestamp,
			  (Generic_debug_level == D_FULL) ? "D_FULL" : "D_BASIC", 
			  timestamp);

	if (write(fd, msg, strlen(msg)) < 0) {
		// if the very first log failed, we disable log
		Generic_logfile[0] = '\0';
	}
	close(fd);
	return;
}

static void
default_log_va(int level, char *fmt, va_list ap)
{
	char str[4000], oldname[PATH_MAX];
	struct tm *t, *localtime();
    time_t clock;
	struct stat statbuf;
	int fd;

	if (!Generic_log_initialized) Generic_logInit(NULL, -1, -1);

	if (Generic_logfile[0] == '\0') return;
	if (level > Generic_debug_level) return;

	// Check the status of logfile
	if (stat(Generic_logfile, &statbuf) < 0) {
		if (errno != ENOENT) {
			// disable log
			Generic_logfile[0] = '\0';
			return;
		}
	}

	// If the logfile is too big, rename the file to file.old
	if (statbuf.st_size > Generic_maxlog) {
		sprintf(oldname, "%s.old", Generic_logfile);
		(void) rename(Generic_logfile, oldname);
	}

	// Make log message
	(void)time(  (time_t *)&clock );
    t = localtime( (time_t *)&clock );
	sprintf(str, "%d/%d %02d:%02d:%02d [%s:%d] ",
			t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec,
			Generic_log_srcfile, Generic_log_lineno );
	vsnprintf(&str[strlen(str)], 4000-strlen(str), fmt, ap);

	// Now open the file and write the message to it
	fd = open(Generic_logfile, O_RDWR | O_CREAT | O_APPEND, S_IRWXU);
	if (fd < 0) {
		return;
	}
	(void) write(fd, str, strlen(str));
	(void) close(fd);
	return;
}

void
Generic_stop_logging()
{
	Generic_log(D_FULL, "GCB logging stopped for process [pid=%ld]\n", 
				(long)getpid());
    Generic_log_enabled = 0;
}

void
Generic_resume_logging()
{
	Generic_log( D_FULL, "GCB logging resumed for process [pid=%ld]\n",
				 (long)getpid() );
    Generic_log_enabled = 1;
}

void
Generic_log(int level, char *fmt, ...)
{
    int saved_errno = errno;
    va_list ap;
    if (!Generic_log_enabled) { return; }
    va_start(ap, fmt);
    (*Generic_log_va)(level, fmt, ap);
    va_end(ap);
    errno = saved_errno;
}


static void
Generic_error_va(char *fmt, va_list ap, int saved_errno)
{
	char str[4000];

	vsnprintf(str, 4000, fmt, ap);
    if (saved_errno != 0) {
		Generic_log(D_BASIC, "ERROR \"%s\" errno %d: %s\n", 
					str, saved_errno, strerror(saved_errno));
	} else {
		Generic_log(D_BASIC, "ERROR \"%s\"\n", str); 
    }

	return;
}


void
Generic_error(char *fmt, ...)
{
	int saved_errno = errno;
	va_list ap;

	va_start(ap, fmt);
	Generic_error_va( fmt, ap, saved_errno );
	va_end(ap);
	errno = saved_errno;
}


void
Generic_abort(char *fmt, ...)
{
	int saved_errno = errno;
    va_list ap;
	va_start(ap, fmt);
	Generic_error_va( fmt, ap, saved_errno );
	va_end(ap);
	Generic_log(D_BASIC, "PID %ld EXITING with failure (status 1)\n",
				(long)getpid() );
	exit( 1 );
}


void
Generic_job_log(char *msg)
{
    char str[4000];
    struct tm *t, *localtime();
    time_t clock;
    int fd;

    // Make log message
    (void)time(  (time_t *)&clock );
    t = localtime( (time_t *)&clock );
    sprintf(str, "%d/%d %02d:%02d:%02d [%s:%d] ",
            t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec,
            Generic_log_srcfile, Generic_log_lineno );
    sprintf(&str[strlen(str)], msg);

    // Now open the file and write the message to it
    fd = open("/tmp/Genericlog", O_RDWR | O_CREAT | O_APPEND, S_IRWXU|S_IRWXG|S_IRWXO);
    if (fd < 0) {
        return;
    }
    (void) write(fd, str, strlen(str));
    (void) close(fd);
    return;
}




#if 0
/* This should work like read_with_timeout, but for write()

TODO: Implement timeout. :-/
*/
static int write_with_timeout(int fd, const char * buf, size_t bytes, struct timeval timelimit) {
	/* TODO implement timeout */
	while(1) {
		ssize_t ret = write(fd, buf, bytes);
		if(ret == bytes) { return 1; }
		if(ret > 0) {
			bytes -= ret;
			buf += ret;
			continue;
		}
		switch(errno) 
		{
			case EAGAIN:
			case EINTR:
				continue;
		}
		return 0;
	}

	/* Can't happen */
	return 0;
}
#endif


static char * http_log_host = NULL;
static int http_log_port = 0;
static char * http_log_path = NULL;
static int http_log_fd = 0;
static struct sockaddr_in http_log_address;

static void shutdown_http_log() {
	free(http_log_host);
	http_log_host = 0;

	http_log_port = 0;

	free(http_log_path);
	http_log_path = 0;
}

static int initialize_http_log() {
	const char * envname = "GCB_HTTP_LOG_URL";
	char * env;
	size_t envlen;
	char * envend;
	char * portstr;
	char * pathstr;
	char * tmpsinful;
	if(http_log_host != NULL) { return 1; }  /* already done */

	env = getenv(envname);
	if(env == NULL) { return 1; } /* No log requested. OK. */

	envlen = strlen(env);
	envend = env + envlen;

	if(envlen < 14) {
		LOG(D_BASIC, "Could not parse %s=%s (too short)\n", envname, env);
		return 0;
	}
	if(memcmp(env, "http://", 7) != 0) {
		LOG(D_BASIC, "Could not parse %s=%s (does not begin with http://)\n", envname, env);
		return 0;
	}
	env += 7;

	portstr = strchr(env, ':');
	pathstr = strchr(env, '/');
	if(portstr && pathstr && portstr > pathstr) { portstr = NULL; }

	if(pathstr != NULL) {
		size_t len = envend - pathstr + 1;
		http_log_path = (char*)malloc(len);
		memcpy(http_log_path, pathstr, len);
		http_log_path[len] = 0;
	} else {
		http_log_path = strdup("/");
		pathstr = envend;
	}

	if(portstr != NULL) {
		http_log_port = atoi(portstr+1);
		if(http_log_port == 0) {
			LOG(D_BASIC, "Could not parse %s=%s (Failed to parse port from '%s')\n", envname, env, portstr);
			return 0; 
		}
	} else {
		http_log_port =  80;
		portstr = pathstr;
	}

	{
		size_t len = portstr - env;
		http_log_host = (char*)malloc(len+1);
		memcpy(http_log_host, env, len);
		http_log_host[len] = 0;
	}

	tmpsinful = malloc(strlen(http_log_host) + strlen("<:99999>") + 1);
	sprintf(tmpsinful, "<%s:%d>", http_log_host, http_log_port);
	if( _str2sin(tmpsinful, &http_log_address) != 0 ) {
		free(http_log_host);
		http_log_host = NULL;
		http_log_port = 0;
		free(http_log_path);
		http_log_path = NULL;
	}
	free(tmpsinful);

	return http_log_host != NULL;
}


static void http_log_close_socket() {
	close(http_log_fd); 
	http_log_fd = 0;
}

static int http_log_open_socket() {
	int ret;
	if(http_log_host == NULL) {
		return 0; /* Not initialized */
	}
	if(http_log_fd) {
		http_log_close_socket();
	}

	http_log_fd = socket(AF_INET, SOCK_STREAM, 0);
	if(http_log_fd == -1) {
		LOG(D_BASIC, "Failed to create socket. %d\n", errno);
		return 0;
	}

	ret = connect(http_log_fd, (struct sockaddr*)&http_log_address, sizeof(struct sockaddr_in));
	if( ret != 0 ) {
		LOG(D_BASIC, "Failed to connect. %d!=0. %d\n", ret, errno);
		close(http_log_fd);
		return 0;
	}

	return 1;
}

static int http_log(const char * msg) {
	const char * HTTP_GET = "GET ";
	const char * HTTP_VERSION = " HTTP/1.1\n";
	const char * HTTP_HOST = "Host: ";
	const char * HTTP_HOSTEND = "\n\n";
	char http_result[12]; /* "HTTP/1.1 200" NOT NULL TERMINATED! */
	char * buf = 0 ;
	int ret;
	size_t req_len;
	struct timeval timeout;

	timeout.tv_sec = 20;
	timeout.tv_usec = 0;

	if(http_log_host == NULL) {
		return 0; /* Not initialized */
	}

	/* We're being lazy and not bothering to parse the response.  (I'm putting
	 * it off because chunked transfer is a nuisance to parse.)  As such we
	 * can't reuse the socket.  So re-open it every time. */
	if( ! http_log_open_socket() ) {
		return 0;
	}

	if(msg == NULL) { return 0; }

	req_len = strlen(HTTP_GET) + strlen(http_log_path) + strlen(msg) + strlen(HTTP_VERSION) + strlen(HTTP_HOST) + strlen(http_log_host) + strlen(HTTP_HOSTEND) + 1;
	buf = (char *)malloc(req_len);
	if( ! buf ) {
		return 0;
	}

	sprintf(buf, "%s%s%s%s%s%s%s", HTTP_GET, http_log_path, msg, HTTP_VERSION, HTTP_HOST, http_log_host, HTTP_HOSTEND);

	ret = write_with_timeout(http_log_fd, buf, strlen(buf), timeout);
	free(buf);
	if( ! ret ) {
		LOG(D_BASIC, "Failed to write. %d\n", errno);
		return 0;
	}

	if( ! read_with_timeout(http_log_fd, http_result, sizeof(http_result), timeout) ) {
		return 0;
	}

	if(memcmp(http_result, "HTTP/", 5) != 0) {
		return 0;
	}

	return 1;
}




uint32_t GL_IP_NA()
{
	static uint32_t result = 0;
	if(result == 0) {
		struct in_addr address;

		/* This is a highly arbitrary, randomly selected address from the
		 * private RFC1918 space.  While potentially valid, it should be
		 * generally unlikely to actually occur, so we're using it as a "not
		 * available" IP address distinct from 0.0.0.0
		*/

		if(inet_aton("10.31.141.156", &address)) {
			result = address.s_addr;
		}
	}
	return result;
}


static char * gcb_task = 0;
void gcb_log_set_task(const char * task) {
	if (gcb_task) {
		free(gcb_task);
		gcb_task = NULL;
	}
	if (task) {
		gcb_task = strdup(task);
	}
}
const char * gcb_log_get_task() { return gcb_task; }

static uint32_t gcb_my_ip_addr;
void gcb_log_set_my_ip_addr(uint32_t addr) { gcb_my_ip_addr = addr; }
uint32_t gcb_log_get_my_ip_addr() { return gcb_my_ip_addr; }

static uint32_t gcb_broker_ip_addr;
void gcb_log_set_broker_ip_addr(uint32_t addr) { gcb_broker_ip_addr = addr; }
uint32_t gcb_log_get_broker_ip_addr() { return gcb_broker_ip_addr; }


/* sizeof(dst) should be sizeof(src)*3 or else you risk truncation.
returns 1 on success, 0 on truncation.
*/
int url_escape(char * dst, size_t dstlen, const char * src) {
	char * dstend = dst + dstlen;
	static int initialized = 0;
	static char is_safe[256];
	int i;
	if( ! initialized ) {
		/* What's "safe"?  A-Za-z0-9_.!~*'()- according to RFC 2396 2.3 */
		initialized = 1;
		for(i = 0; i < sizeof(is_safe); i++) {
			is_safe[i] = (i >= 'a' && i <= 'z') ||
				(i >= 'A' && i <= 'Z') ||
				(i >= '0' && i <= '9');
		}
		is_safe['_'] = 1;
		is_safe['.'] = 1;
		is_safe['-'] = 1;
		is_safe['!'] = 1;
		is_safe['~'] = 1;
		is_safe['*'] = 1;
		is_safe['\''] = 1;
		is_safe['('] = 1;
		is_safe[')'] = 1;
	}

	if( src == NULL ) {
		*dst = 0;
		return 1;
	}

	while(*src) {
		if((dst+3) >= dstend) {
			/* Not at least 3 spaces left? Give up, we might not fit. */
			return 0;
		}
		if( is_safe[(int)*src] ) {
			*dst = *src;
			dst++;
		} else {
			sprintf(dst, "%%%02X", (unsigned int) *src);
			dst += 3;
		}
		src++;
	}
	*dst = 0;
	return 1;
}

void
gcb_log_impl(
	unsigned int msgid,

	const char * filename,
	int lineno,

	const char * subsystem,

	unsigned char severity,

	const char * functionname,
	const char * gcb_task,

	const char * msg,

	uint32_t my_ip_addr,
	uint32_t my_port,

	uint32_t broker_ip_addr,
	uint32_t relay_server_port,

	uint32_t remote_ip_addr,
	uint32_t remote_port
	)
{
	const char * tmp;
	char url[4096];
	size_t index = 0;
	size_t urllen = sizeof(url);
	if( ! initialize_http_log() ) {
		return;
	}

		/* truncate filename to just filename */
	if(filename) {
		tmp = strrchr(filename, '/');
		if(tmp) { filename = tmp; }
	}

	snprintf(url + index, urllen - index, "%s", "?filename=");
	index += strlen(url+index);
	url_escape(url + index, urllen-index, filename);
	index += strlen(url+index);

	snprintf(url + index, urllen - index, "&linenum=%d", lineno);
	index += strlen(url+index);

	snprintf(url + index, urllen - index, "&msgid=%u", msgid);
	index += strlen(url+index);

	snprintf(url + index, urllen - index, "%s", "?subsystem=");
	index += strlen(url+index);
	url_escape(url + index, urllen-index, subsystem);
	index += strlen(url+index);

	snprintf(url + index, urllen - index, "&msgid=%u", (unsigned int)severity);
	index += strlen(url+index);

	snprintf(url + index, urllen - index, "%s", "?function=");
	index += strlen(url+index);
	url_escape(url + index, urllen-index, functionname);
	index += strlen(url+index);

	snprintf(url + index, urllen - index, "%s", "?gcb_task=");
	index += strlen(url+index);
	url_escape(url + index, urllen-index, gcb_task);
	index += strlen(url+index);

	snprintf(url + index, urllen - index, "%s", "?msg=");
	index += strlen(url+index);
	url_escape(url + index, urllen-index, msg);
	index += strlen(url+index);

	/* TODO: Add IP addrs and port. special case NA values.*/
	http_log(url);


#if 0
	/* Crude testing stub; itself untested */
	fprintf(stderr, "%d [%s:%d] %d %s/%s: %s", msgid, filename?filename:"(unknown file)", lineno, (int) severity,
		task?task:"(unknown task)", functionname?functionname:"(unknown function)", msg);
#endif
	return;
}




