/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#ifndef GCBSTRING_H
#define GCBSTRING_H

/* A simple string library for GCB.  What little functionality it has is
 * roughly designed to look like std::string, but designed in C for
 * portability.  */

/* Thou shalt not go digging around inside this structure!  Treat GCBString as
 * opaque! */
typedef struct GCBStringI {
	char * s;
	size_t strlen;
	size_t buflen;
} * GCBString;

/* Create a new GCBString. Be sure to GCB_string_delete it or you'll leak
 * memory! */
GCBString GCB_string_new(const char * p);

/* Create a new, empty GCBString.  bufsize is the size of the buffer to
 * initially allocate.  0 is okay as a default (albet slightly inefficient). */
GCBString GCB_string_new_empty(size_t bufsize);

/* Free memory associated with this GCBString. String becomes invalid and
 * should not be used after this point! */
void GCB_string_delete(GCBString s);

/* Empties the string, making it zero length.  Note that the buffer associated
 * with the string is kept around. This is more efficient than delete/new if
 * you need a "scratch string" that you're going to reuse a bunch of times.  */
void GCB_string_truncate(GCBString s);

/* As strlen or std::string::length/size */
size_t GCB_string_length(GCBString s);

/* Get a C style string.  No mucking about with it.  */
const char * GCB_string_c_str(GCBString s);

/* Delete last character */
void GCB_string_trim_last(GCBString s);

/* Like strcat or std::string::append. Variations are overloading.  */
void GCB_string_appendGG(GCBString dest, GCBString src);
void GCB_string_appendGP(GCBString dest, const char * src);

#endif /*GCBSTRING_H*/
