/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#ifndef _Generic_Sock_H
#define _Generic_Sock_H

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#ifdef __cplusplus
extern "C" {
#endif

int useGeneric(void);
int Generic_socket(int domain, int type, int protocol);
int Generic_bind(int fd, struct sockaddr *my_addr, socklen_t addrlen);
int Generic_listen(int fd, int backlog);
int Generic_connect(int fd, const struct sockaddr *serv_addr, socklen_t addrlen);
int Generic_accept(int fd, struct sockaddr * peer, socklen_t *addrlen);
int Generic_select(
		int maxfd,
		fd_set *readfds,
		fd_set *writefds,
		fd_set *exceptfds,
		struct timeval *timeout);
int Generic_recv(int fd, void *buf, size_t len, int flags);
int Generic_read(int fd, void *buf, size_t count);
int Generic_recvfrom(
		int fd,  void *buf, size_t len, int flags,
		struct sockaddr *from, socklen_t *fromlen);
int Generic_send(int fd, const void *msg, size_t len, int flags);
int Generic_sendto(
		int fd,
		const void *msg, size_t len, int flags,
		const struct sockaddr *to, socklen_t tolen);
int Generic_write(int fd, const void *buf, size_t count);
int Generic_close(int fd);
int Generic_dup(int fd);
int Generic_dup2(int oldfd, int newfd);
int Generic_getsockname(int fd, struct sockaddr * name, socklen_t *namelen);
int Generic_getsockopt(
		int fd,
		int level,
		int optname,
		void *opt_val,
		socklen_t *optlen);
pid_t Generic_fork(void);
int Generic_execv(const char *path, char *const argv[]);
int Generic_execl(const char *path, const char *arg, ...);
int Generic_execve(const char *path, char *const argv [], char *const envp[]);
void Generic_cleanup(void);
void Generic_set_err_callback(void (*app_callback)(void), int timeout);

#ifdef __cplusplus
}
#endif

#endif // #ifndef _USE_Generic_H
