/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include <iostream>
#include "GCB.h"

#define TCP_BUF_SIZE 100000

extern "C" {
char *_sin2str(const struct sockaddr_in *sin);
}

using namespace std;

void echoUDP(int fd)
{
	char buf[1040];
	struct sockaddr_in from;
	socklen_t fromlen = sizeof(from);

	int read = GCB_recvfrom(fd,  buf, sizeof(buf), 0, (struct sockaddr *)&from, &fromlen);
	if (read < 0) {
		cerr << "_CB_recvfrom failed\n";
		return;
	}

	int sent = GCB_sendto(fd, buf, read, 0, (struct sockaddr *)&from, fromlen);
	if (sent < 0) {
		cerr << "GCB_sendto failed\n";
	}
	return;
}

void my_callback(void)
{
    exit(-1);
}

int main(int argc, char *argv[])
{
	int tcpsock = 20;
	int udpsock = 30;
	int maxfd = udpsock + 1;
	cout << "EchoServer started\n";

	struct sockaddr_in sin;
	socklen_t sinlen = sizeof(sin);
	if (GCB_getsockname(tcpsock, (struct sockaddr *)&sin, &sinlen) < 0) {
		cerr << "getsockname failed: " << strerror(errno) << endl;
		exit(-1);
	}
	cout << "server TCP socket: " << _sin2str(&sin) << endl;
	if (GCB_getsockname(udpsock, (struct sockaddr *)&sin, &sinlen) < 0) {
		cerr << "getsockname failed: " << strerror(errno) << endl;
		exit(-1);
	}
	cout << "server UDP socket: " << _sin2str(&sin) << endl;

    // Register callback function for extended Broker failure
    GCB_Broker_down_callback_set(my_callback, 90);

	while(1) {
		fd_set rdfds;
		FD_ZERO(&rdfds);
		FD_SET(tcpsock, &rdfds);
		FD_SET(udpsock, &rdfds);
		int ready = GCB_select(maxfd, &rdfds, NULL, NULL, NULL);
		if (ready < 0) {
            if (errno != EINTR) { continue; }
			perror("select");
			exit(-1);
		}

		if (FD_ISSET(udpsock, &rdfds)) {
			echoUDP(udpsock);
		}

		if (FD_ISSET(tcpsock, &rdfds)) {
			// accept the connection
			struct sockaddr_in from;
			socklen_t addrLen = sizeof(from);
			int newfd = GCB_accept(tcpsock, (struct sockaddr *)&from, &addrLen);
			if (newfd < 0) {
				cerr << "Child process can't accept the connection\n";
				exit(-1);
			}
			pid_t pid = fork();
			if (pid < 0) {
				cerr << "fork error\n";
				exit(-1);
			} else if (pid == 0) { // child
				// dup the connection to fd = 0
				GCB_dup2(newfd, 0);
				// close all fds except fd = 0
				for (int i=1; i<1024; i++) {
					GCB_close(i);
				}
				// execve to TcpEcho
				char arg0[20];
				char *TCPecho_argv[2];
				sprintf(arg0, "./TCPecho");
				TCPecho_argv[0] = arg0;
				TCPecho_argv[1] = NULL;
				GCB_execv("./TCPecho", TCPecho_argv);
				exit(-1);
			} else { // parent
				GCB_close(newfd);
			}
		}
	}
}
