/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"

// run as "multi_client server1_ip server2_ip server3_ip port concurrency loop"
int
main(int argc, char *argv[])
{
    if (argc != 7) {
        printf("usage: multi_client server1_ip server2_ip server3_ip port concurrency loop\n");
        return -1;
    }

    // stash servers' addresses
    struct sockaddr_in peer1, peer2, peer3, *peer;
    peer1.sin_family = AF_INET;
    if (inet_aton(argv[1], &peer1.sin_addr) < 0) {
        printf("server1_ip is in invalid form: %s\n", strerror(errno));
        return -1;
    }
    peer2.sin_family = AF_INET;
    if (inet_aton(argv[2], &peer2.sin_addr) < 0) {
        printf("server2_ip is in invalid form: %s\n", strerror(errno));
        return -1;
    }
    peer3.sin_family = AF_INET;
    if (inet_aton(argv[3], &peer3.sin_addr) < 0) {
        printf("server3_ip is in invalid form: %s\n", strerror(errno));
        return -1;
    }
    short port = atoi(argv[4]);
    peer1.sin_port = peer2.sin_port = peer3.sin_port = htons(port);
    socklen_t addrlen = sizeof(struct sockaddr_in);

    int concurrency = atoi(argv[5]);
    int noexp = atoi(argv[6]);

    int exp = 0;
    int turn = 0;
    printf("=======================================\n");
    printf("  concur   avg. time (%d exp)\n", exp);
    printf("=======================================\n");
    while (exp < noexp) {
        for (int loop=1; loop <= concurrency; loop++) {
            // flush fds
            bool fds[5000];
            for (int i=0; i<5000; i++) { fds[i] = false; }
            // record time
            struct timeval before;
            gettimeofday(&before, NULL);
            // do nonblock connect $(loop) times
            for (int i=0; i<loop; i++) {
                // create a nonblocking socket
                int fd = GCB_socket(AF_INET, SOCK_STREAM, 0);
                if (fd < 0) {
                    printf("GCB_socket failed: %s\n", strerror(errno));
                    return -1;
                }
                int val;
                if ((val = fcntl(fd, F_GETFL, 0)) < 0) {
                    printf("fcntl failed - %s\n", strerror(errno));
                    return -1;
                }
                if (fcntl(fd, F_SETFL, (void *)(val | O_NONBLOCK)) < 0) {
                    printf("fcntl failed - %s\n", strerror(errno));
                    return -1;
                }
                fds[fd] = true;
                // connect to the peer of the next turn
                if (turn == 0) { peer = &peer1; }
                else if (turn == 1) { peer = &peer2; }
                else { peer = &peer3; }
                turn = (++turn) % 3;
                if (GCB_connect(fd, (struct sockaddr *)peer, addrlen) < 0) {
                    if (errno != EINPROGRESS) {
                        printf("GCB_connect: failed - %s\n", strerror(errno));
                        return -1;
                    }
                }
            }
            // wait until all connections done
            int completed = 0;
            while (completed < loop) {
                fd_set wrfds;
                int maxfd = 0;
                FD_ZERO(&wrfds);
                for (int i=0; i<5000; i++) { if (fds[i]) { FD_SET(i, &wrfds); maxfd = i+1; }}
                int ready = GCB_select(maxfd, NULL, &wrfds, NULL, NULL);
                if (ready < 0) {
                    if (errno == EINTR) continue;
                    printf("GCB_select failed: %s\n", strerror(errno));return -1;
                }
                if (ready > 0) {
                    for (int i=0; i<maxfd; i++) {
                        if (FD_ISSET(i, &wrfds)) {
                            int error;
                            socklen_t len = sizeof(error);
                            if (GCB_getsockopt(i, SOL_SOCKET, SO_ERROR, &error, &len) < 0) {
                                printf("<tcp_client_2> GCB_getsockopt failed: %s\n", strerror(errno));
                                return -1;
                            }
                            if (error) {
                                printf("<tcp_client_2> GCB_getsockopt returned: %s\n", strerror(error));
                                return -1;
                            }
                            GCB_close(i);
                            completed++;
                            fds[i] = false;
                        }
                    }
                }
            }
            // record elapsed time
            struct timeval after;
            gettimeofday(&after, NULL);
            long int elapsed = (after.tv_sec - before.tv_sec) * 1000000 + (after.tv_usec - before.tv_usec);
            printf("%d %ld\n", loop, elapsed);
        }
        exp++;
        sleep(5);
    }

    printf("---------------------------------------\n");
}
