/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"

int
main(int argc, char *argv[])
{
    if (argc != 4) {
        printf("usage: tcp_client_1 port server_ip server_port\n");
        return -1;
    }

    int fd = GCB_socket(AF_INET, SOCK_STREAM, 0);
    if (fd < 0) {
        printf("GCB_socket failed: %s\n", strerror(errno));
        return -1;
    }

    short client_port = atoi(argv[1]);
    struct sockaddr_in myaddr;
    bzero(&myaddr, sizeof(myaddr));
    myaddr.sin_family = AF_INET;
    myaddr.sin_addr.s_addr = htonl(INADDR_ANY);
    myaddr.sin_port = htons(client_port);
    if (GCB_bind(fd, (struct sockaddr *)&myaddr, sizeof(myaddr)) < 0) {
        printf("GCB_bind failed: %s\n", strerror(errno));
        return -1;
    }

    struct sockaddr_in peer;
    peer.sin_family = AF_INET;
    if (inet_aton(argv[2], &peer.sin_addr) < 0) {
        printf("ip address is in invalid form: %s\n", strerror(errno));
        return -1;
    }
    short server_port = atoi(argv[3]);
    peer.sin_port = htons(server_port);
    socklen_t addrlen = sizeof(struct sockaddr_in);
    if (GCB_connect(fd, (struct sockaddr *)&peer, addrlen) < 0) {
        printf("GCB_connect: failed - %s\n", strerror(errno));
        return -1;
    }

    while(1) {
        printf("***type string send: ");
        char sbuf[500];
        (void) gets(sbuf);
        if (sbuf[0] == '!') {
            GCB_close(fd);
            return 0;
        }
        int sent = GCB_send(fd, sbuf, strlen(sbuf)+1, 0);
        printf("***%d bytes sent\n", sent);
        int total = 0, rcved = 0;
        char rbuf[500];
        while(total < sent) {
            rcved = GCB_recv(fd, rbuf, sent - total, 0);
            if (rcved < 0) {
                printf("GCB_recv failed: %s\n", strerror(errno));
                GCB_close(fd);
                return -1;
            } else if (rcved == 0) {
                printf("server closed\n");
                GCB_close(fd);
                return 0;
            }
            total += rcved;
        }
        printf("%d bytes received: %s\n", total, rbuf);
        for (int i=0; i<total; i++) {
            if (sbuf[i] != rbuf[i]) {
                printf("compare failed\n");
                GCB_close(fd);
                return -1;
            }
        }
    }
}
