/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"

// run as "tcp_server_1 port"
int
main(int argc, char *argv[])
{
    if (argc != 2) {
        printf("usage: tcp_server_1 port\n");
        return -1;
    }

    int fd = GCB_socket(AF_INET, SOCK_STREAM, 0);
    if (fd < 0) {
        printf("GCB_socket failed: %s\n", strerror(errno));
        return -1;
    }

    short port = atoi(argv[1]);
    uint16_t nport = htons(port);
    struct sockaddr_in myaddr;
    myaddr.sin_family = AF_INET;
    myaddr.sin_addr.s_addr = htonl(INADDR_ANY);
    myaddr.sin_port = nport;
    if (GCB_bind(fd, (struct sockaddr *)&myaddr, sizeof(myaddr)) < 0) {
        printf("GCB_bind failed: %s\n", strerror(errno));
        return -1;
    }
    if (GCB_listen(fd, 5) < 0) {
        printf("GCB_listen: failed - %s\n", strerror(errno));
        return -1;
    }
    socklen_t addrlen = sizeof(struct sockaddr_in);
    if (GCB_getsockname(fd, (struct sockaddr *)&myaddr, &addrlen) < 0) {
        printf("GCB_getsockname failed - %s\n", strerror(errno));
        return -1;
    }
    printf("listening on %s\n", _sin2str(&myaddr));

    struct sockaddr_in peer;
    addrlen = sizeof(struct sockaddr_in);
    int newfd = GCB_accept(fd, (struct sockaddr *)&peer, &addrlen);
    if (newfd < 0) {
        printf("GCB_accept: failed - %s\n", strerror(errno));
        return -1;
    }
    printf("connection accepted from %s\n", _sin2str(&peer));
    printf("closing the listening socket\n");
    GCB_close(fd);

    while(1) {
        char buf[1000];
        int bytes = GCB_recv(newfd, buf, 1000, 0);
        if (bytes < 0) {
            printf("GCB_recv failed: %s\n", strerror(errno));
            return -1;
        } else if (bytes == 0) {
            GCB_close(newfd);
            printf("done\n");
            return 0;
        }
        printf("\t%d bytes received\n", bytes);
        int total = 0, sent = 0;
        while(total < bytes) {
            sent = GCB_send(newfd, &buf[total], bytes - total, 0);
            if (sent < 0) {
                printf("GCB_send failed: %s\n", strerror(errno));
                return -1;
            }
            total += sent;
        }
        printf("\t%d bytes sent\n", total);
    }
}
