/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"

static int
echo(int fd)
{
    int bytes;
    int total, sent;
    char buf[10000];

    bytes = GCB_recv(fd, buf, 10000, 0);
    if (bytes < 0) {
        printf("GCB_recv failed: %s\n", strerror(errno));
        return -1;
    } else if (bytes == 0) {
        printf("client closed\n");
        return 0;
    }
    printf("\t%d bytes received\n", bytes);
    total = sent = 0;
    while(total < bytes) {
        sent = GCB_send(fd, &buf[total], bytes - total, 0);
        if (sent < 0) {
            printf("GCB_send failed: %s\n", strerror(errno));
            return -1;
        }
        total += sent;
    }
    printf("\techoed\n");
    return total;
}

// run as "tcp_server_1 port"
int
main(int argc, char *argv[])
{
    int fd, newfd, maxfd;
    short port;
    struct sockaddr_in myaddr;
    fd_set rdfds;
    int ready;
    char fds[1000];
    int i;

    for (i=0; i<1000; i++) {
        fds[i] = 0;
    }

    if (argc != 2) {
        printf("usage: tcp_server_1 port\n");
        return -1;
    }

    fd = GCB_socket(AF_INET, SOCK_STREAM, 0);
    if (fd < 0) {
        printf("GCB_socket failed: %s\n", strerror(errno));
        return -1;
    }
    printf("listen sock: fd = %d\n", fd);

    myaddr.sin_family = AF_INET;
    myaddr.sin_addr.s_addr = htonl(INADDR_ANY);
    port = atoi(argv[1]);
    myaddr.sin_port = htons(port);
    if (GCB_bind(fd, (struct sockaddr *)&myaddr, sizeof(myaddr)) < 0) {
        printf("GCB_bind failed: %s\n", strerror(errno));
        return -1;
    }
    if (GCB_listen(fd, 5) < 0) {
        printf("GCB_listen: failed - %s\n", strerror(errno));
        return -1;
    }
    socklen_t addrlen = sizeof(struct sockaddr_in);
    if (GCB_getsockname(fd, (struct sockaddr *)&myaddr, &addrlen) < 0) {
        printf("GCB_getsockname failed - %s\n", strerror(errno));
        return -1;
    }
    printf("apparently listening on %s\n", _sin2str(&myaddr));

	addrlen = sizeof(struct sockaddr_in);
	if(GCB_real_getsockname(fd, (struct sockaddr *)&myaddr, &addrlen) < 0 ) {
        printf("GCB_real_getsockname failed - %s\n", strerror(errno));
        return -1;
	}
    printf("actually listening on %s\n", _sin2str(&myaddr));

    fds[fd] = 1;

    while(1) {
        // select on listen sock and accepted socks
        FD_ZERO(&rdfds);
        maxfd = 0;
        for (i=0; i<1000; i++) {
            if (fds[i]) {
                FD_SET(i, &rdfds);
                maxfd = i + 1;
            }
        }
        ready = GCB_select(maxfd, &rdfds, NULL, NULL, NULL);
        if (ready < 0) {
            if (errno == EINTR) continue;
            printf("GCB_select failed: %s\n", strerror(errno));
            return -1;
        }
        for (i=0; i<maxfd; i++) {
            if (i == fd) continue;
            if (FD_ISSET(i, &rdfds)) {
                if (echo(i) <= 0) {
                    GCB_close(i);
                    fds[i] = 0;
                }
            }
        }
        if (FD_ISSET(fd, &rdfds)) {
            struct sockaddr_in peer;
            addrlen = sizeof(peer);
            newfd = GCB_accept(fd, (struct sockaddr *)&peer, &addrlen);
            if (newfd < 0) {
                printf("GCB_accept: failed - %s\n", strerror(errno));
                return -1;
            }
            printf("connection accepted from %s\n", _sin2str(&peer));
            fds[newfd] = 1;
        }
    }
}
