/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"

// run as "tcp_server_3 port"
int
main(int argc, char *argv[])
{
    if (argc != 2) {
        printf("usage: tcp_server_3 port\n");
        return -1;
    }

    int fd = GCB_socket(AF_INET, SOCK_STREAM, 0);
    if (fd < 0) {
        printf("GCB_socket failed: %s\n", strerror(errno));
        return -1;
    }
    printf("listen sock: fd = %d\n", fd);

    struct sockaddr_in myaddr;
    myaddr.sin_family = AF_INET;
    myaddr.sin_addr.s_addr = htonl(INADDR_ANY);
    short port = atoi(argv[1]);
    myaddr.sin_port = htons(port);
    if (GCB_bind(fd, (struct sockaddr *)&myaddr, sizeof(myaddr)) < 0) {
        printf("GCB_bind failed: %s\n", strerror(errno));
        return -1;
    }
    if (GCB_listen(fd, 5) < 0) {
        printf("GCB_listen: failed - %s\n", strerror(errno));
        return -1;
    }
    socklen_t addrlen = sizeof(struct sockaddr_in);
    if (GCB_getsockname(fd, (struct sockaddr *)&myaddr, &addrlen) < 0) {
        printf("GCB_getsockname failed - %s\n", strerror(errno));
        return -1;
    }
    printf("listening on %s\n", _sin2str(&myaddr));

    while(1) {
        struct sockaddr_in peer;
        addrlen = sizeof(peer);
        int newfd = GCB_accept(fd, (struct sockaddr *)&peer, &addrlen);
        if (newfd < 0) {
            printf("GCB_accept: failed - %s\n", strerror(errno));
            return -1;
        }
        printf("connection accepted from %s\n", _sin2str(&peer));
        printf("forking a child process...\n");
        pid_t pid = fork();
        if (pid == 0) {
            char arg1[10];
            sprintf(arg1, "%d", fd);
            char arg2[10];
            sprintf(arg2, "%d", newfd);
            GCB_execl("./tcp_echo_server", "tcp_echo_server", arg1, arg2, NULL); 
        } else {
            GCB_close(newfd);
        }
    }
}
