/***************************************************************
 *
 * Copyright (C) 1990-2007, Condor Team, Computer Sciences Department,
 * University of Wisconsin-Madison, WI.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you
 * may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ***************************************************************/

#include "gcb_common.h"
#include "GCB.h"

// run as "udp_server_2 port"
int
main(int argc, char *argv[])
{
    int fd;
    short port;
    char buf[1000];
    int bytes, sent;
    fd_set rdfds;
    int ready;

    if (argc != 2) {
        printf("usage: udp_server_2 port\n");
        return -1;
    }

    fd = GCB_socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0) {
        printf("GCB_socket failed: %s\n", strerror(errno));
        return -1;
    }
    printf("listen sock: fd = %d\n", fd);

    struct sockaddr_in myaddr;
    myaddr.sin_family = AF_INET;
    myaddr.sin_addr.s_addr = htonl(INADDR_ANY);
    port = atoi(argv[1]);
    myaddr.sin_port = htons(port);
    if (GCB_bind(fd, (struct sockaddr *)&myaddr, sizeof(myaddr)) < 0) {
        printf("GCB_bind failed: %s\n", strerror(errno));
        return -1;
    }

    socklen_t addrlen = sizeof(struct sockaddr_in);
    if (GCB_getsockname(fd, (struct sockaddr *)&myaddr, &addrlen) < 0) {
        printf("GCB_getsockname failed - %s\n", strerror(errno));
        return -1;
    }
    printf("apparently listening on %s\n", _sin2str(&myaddr));

    if (GCB_real_getsockname(fd, (struct sockaddr *)&myaddr, &addrlen) < 0) {
        printf("GCB_real_getsockname failed - %s\n", strerror(errno));
        return -1;
    }
    printf("actually   listening on %s\n", _sin2str(&myaddr));

    while(1) {
        FD_ZERO(&rdfds);
        FD_SET(fd, &rdfds);
        ready = GCB_select(fd+1, &rdfds, NULL, NULL, NULL);
        if (ready < 0) {
            if (errno == EINTR) { continue; }
            printf("select failed: %s\n", strerror(errno));
            return -1;
        }
        struct sockaddr_in from;
        socklen_t fromlen = sizeof(from);
        bytes = GCB_recvfrom(fd, buf, 1000, 0,
                (struct sockaddr *)&from, &fromlen);
        if (bytes < 0) {
            printf("GCB_recvfrom failed: %s\n", strerror(errno));
            return -1;
        }
        printf("\t%d bytes received from: %s\n", bytes, _sin2str(&from));
        printf("\tcontent: %s\n", buf);
        sent = GCB_sendto(fd, buf, bytes, 0,
            (struct sockaddr *)&from, fromlen);
        if (sent < 0) {
            printf("GCB_sendto failed: %s\n", strerror(errno));
            return -1;
        }
        printf("\treplied: %s\n", buf);
    }
}
